
// ------------------------------
//  base class definition

#include "cvlib.h"


// ------------------------------------------------------------------
// Define the actual library objects

//#include "cvlib_cv.h"             // dummy object for cvlib instantiation

#include "cvlib_image.h"           // loads an image
#include "cvlib_show_image.h"       // displays an image
//#include "cvlib_import_gem_pix.h"    // bridge from gem to cv
//#include "cvlib_export_gem_pix.h"

//#include "cvlib_copy.h"
#include "cvlib_set_roi.h"

#include "cvlib_canny.h"
#include "cvlib_dilate.h"
#include "cvlib_erode.h"
#include "cvlib_median.h"
#include "cvlib_gaussian.h"
#include "cvlib_athreshold.h"

#include "cvlib_circle.h"

#include "cvlib_find_contours.h"
#include "cvlib_approx_poly.h"
#include "cvlib_draw_contours.h"
#include "cvlib_contours_convexity2.h"
#include "cvlib_contours_convexity_defects.h"
#include "cvlib_contours_convex_hull.h"

//#include "cvlib_convex_hull.h"
//#include "cvlib_match_contours.h"
#include "cvlib_hough_circles.h"


#include "pix_opencv/pix_opencv_contours_boundingrect.h"
#include "pix_opencv/pix_opencv_bgsubstract.h"
#include "pix_opencv/pix_opencv_hu_compare.h"
#include "pix_opencv/pix_opencv_threshold.h"
#include "pix_opencv/pix_opencv_contours_convexity.h"

#include "cvlib_contours_segment_count.h"
#include "cvlib_route_contours.h"
#include "cvlib_contours_bounding_rect.h"
#include "cvlib_contours_min_area_rect.h"
#include "cvlib_contours_get.h"
#include "cvlib_contours_hu_moments.h"

//**************************************************************
// global variables


// the array that stores type/status flags and pointers to cvDataStructures

cvlib_pool_struct cvlib_pool[CVLIB_POOL_SIZE];

// index pointing to next free position in array cvlib_pool
int cvlib_blank_index;

// counter for remaining free positions in array
int cvlib_remaining_blank_indexes;

//****************************************************************

// global method called during cvlib setup
void cvlib_reset()
{
    CVLIB_DEBUG("Appel de cvlib_reset()");
    cvlib_remaining_blank_indexes=CVLIB_POOL_SIZE;
    cvlib_blank_index=0;
    int i =0;
    while (i< CVLIB_POOL_SIZE)
    {
        cvlib_pool[i].type = CVLIB_TYPE_NULL;
        cvlib_pool[i].status = CVLIB_STATUS_NULL;
        cvlib_pool[i].pInstance = 0;
        i++;
    }



}

int cvlib_find_next_blank()
{
    if (cvlib_remaining_blank_indexes == 0)
        return 0;   //  failed

        // search for next blank index
    int i=0;
    int found_blank_index = 0;
    while (i< CVLIB_POOL_SIZE)   // anti infinite loop mechanism
        {
            if (CVLIB_POOL_SIZE <= ++cvlib_blank_index)  // check next position
                cvlib_blank_index = 0;
            if (cvlib_pool[cvlib_blank_index].status == CVLIB_STATUS_NULL)  // check for status  (0 = blank)
                 {
                     found_blank_index = 1;
                     break;  // quit the loop
                 }
            i++;
        }
    return found_blank_index;
}

// get pointer to data
void* cvlib_get_pdata(int i)
{
    if ((i>=0) && (i<CVLIB_POOL_SIZE))
        return((void*)cvlib_pool[i].pInstance);
    else
        return 0;
}

// set pointer to data

int cvlib_set_pdata(int index, void* pointer)
{
    if ((index>=0)&&(index<CVLIB_POOL_SIZE))
    {
        if (cvlib_pool[index].status)
        {
            cvlib_pool[index].pInstance=pointer;
            return 1;
        }
    }
    else
        return 0;
}



// get status
int cvlib_get_status(int i)
{
    if ((i>=0) && (i<CVLIB_POOL_SIZE))
        return(cvlib_pool[i].status);
    else return CVLIB_STATUS_ERROR;
}




int cvlib_createHeader(int type,int status)
{

    if (cvlib_find_next_blank())
        {
            cvlib_pool[cvlib_blank_index].pInstance = 0;
            cvlib_pool[cvlib_blank_index].status = status;
            cvlib_pool[cvlib_blank_index].type = type;
            cvlib_remaining_blank_indexes--;
            CVLIB_DEBUG("cvlib_createHeader Success!");
            return cvlib_blank_index;
        }
        else return -1;
}

bool cvlib_isIplImage(int index)
    {
        if ((index>=0) && (index<CVLIB_POOL_SIZE))
           if (cvlib_pool[index].type == CVLIB_TYPE_IPLIMAGE)
                return (true);
        return (false);
    }

int cvlib_get_type(int index)
{
        if ((index>=0) && (index<CVLIB_POOL_SIZE))
           return (cvlib_pool[index].type);
        else
            return (-1);
    }
///  modify
int cvlib_freeHeader(int index)
{
    CVLIB_DEBUG("Appel de cvlib_freeHeader(%d)...", index);
    if ((index < 0) || (index>=CVLIB_POOL_SIZE))
        {
           CVLIB_DEBUG("index %d invalide", index);
        }
    else if (cvlib_pool[index].status != CVLIB_STATUS_NULL)
    {
        cvlib_pool[index].type = CVLIB_TYPE_NULL;
        cvlib_pool[index].status = CVLIB_STATUS_NULL;
        cvlib_pool[index].pInstance = NULL;
        cvlib_remaining_blank_indexes++;
        CVLIB_DEBUG("cvlib_freeHeader = success!");
        return 1;
    }
    else
    {
        CVLIB_DEBUG("cvlib_freeHeader  : position %d already free");
        return 0;
    }



}



    // check if IplImage exist, release and recreate it if necessary
int cvlib_prepareIplImage(IplImage ** pmyimage, int width, int height, int nChannels)
{

    if (*pmyimage==0)
    {

        *pmyimage=cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, nChannels);

        return CVLIB_OK+CVLIB_RESIZED;
    }
    else
    {
        if ((((IplImage *)*pmyimage)->width==width)&&(((IplImage *)*pmyimage)->height==height)&&(((IplImage *)*pmyimage)->nChannels==nChannels))
            return CVLIB_OK;
        else
        {
            cvReleaseImage(pmyimage);
            *pmyimage=cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, nChannels);

            if (*pmyimage!=0)
            {
                return CVLIB_OK+CVLIB_RESIZED;
            }
            else
                return CVLIB_FAILED;
        }
    }
}

// convert colorspace (stored as char[4] in IplImage) into GL_colorspace define value
int cvlib_getIplImageColorspace(IplImage *my_image)
{
    /// TO DO : ADD YUV SUPPORT
        int colorspace;
       /*   GL_RGBA 0x1908
            GL_RGB 0x1907
            GL_BGR 0x80E0
            GL_BGRA 0x80E1
            GL_RGB16 0x8054
            GL_LUMINANCE 0x1909
            GL_LUMINANCE_ALPHA 0x190A
            GL_UNSIGNED_BYTE 0x1401
        */

        if (my_image->colorModel[0]==(char)CVLIB_RGBA)
            colorspace = GL_RGBA;
        else if (my_image->colorModel[0]==(char)CVLIB_BGRA)
            colorspace = GL_BGRA;
        else if (my_image->colorModel[0]==(char)CVLIB_RGB)
            colorspace = GL_RGB;
        else if (my_image->colorModel[0]==(char)CVLIB_BGR)
            colorspace = GL_BGR;
        else if (my_image->colorModel[0]==(char)CVLIB_LUMINANCE)
            colorspace = GL_LUMINANCE;
        else
            switch (my_image->nChannels)
            {
                case 1:
                    colorspace = GL_LUMINANCE;
                    break;
                case 2:
                    colorspace = GL_LUMINANCE_ALPHA;
                    break;
                case 3:
                    colorspace = GL_RGB;
                    break;
                case 4:
                    colorspace = GL_RGBA;
                    break;
                default:
                    colorspace = 0;  /// prevents pix_texture from crash ?
                }

        return colorspace;

}

// convert GL_colorspace into cvlib_colorspace (stored in position colorModel[0] of IplImage struct)
void cvlib_setIplImageColorspace(IplImage *my_image,int gl_colorspace)
{
    /// TO DO : ADD YUV SUPPORT

       /*   GL_RGBA 0x1908
            GL_RGB 0x1907
            GL_BGR 0x80E0
            GL_BGRA 0x80E1
            GL_RGB16 0x8054
            GL_LUMINANCE 0x1909
            GL_LUMINANCE_ALPHA 0x190A
            GL_UNSIGNED_BYTE 0x1401
        */
        switch(gl_colorspace)
        {
            case GL_RGBA:
                my_image->colorModel[0]=(char)CVLIB_RGBA;
                break;
            case GL_BGRA:
                my_image->colorModel[0]=(char)CVLIB_BGRA;
                break;
            case GL_RGB:
                my_image->colorModel[0]=(char)CVLIB_RGB;
                break;
            case GL_BGR:
                my_image->colorModel[0]=(char)CVLIB_BGR;
                break;
            case GL_LUMINANCE:
                my_image->colorModel[0]=(char)CVLIB_LUMINANCE;
            default:
                my_image->colorModel[0]=(char)0;
                break;
        }


}

// ------------------------------------------------
// Do the library setup

static void cvsetup()
{
	post("cvlib : flext-based OpenCV2.0 Pure-Data library,");
	post("cvlib : (c)2010 Jean-Yves Gratius http://jy.gratius.free.fr");
	post("cvlib : Includes Windows port of some pix_opencv objects,");
	post("cvlib : initially written by Yves Degoyon and Lluis Gomez i Bigorda :");
    post("cvlib : see http://www.hangar.org/wikis/lab/doku.php?id=start:puredata_opencv");
	// reset the cvlib_pool structure
    cvlib_reset();


	// call the objects' setup routines


//	FLEXT_SETUP(cvlib_cv);
   FLEXT_SETUP(cvlib_image);
    FLEXT_SETUP(cvlib_show_image);

//    FLEXT_SETUP(cvlib_import_gem_pix);
//    FLEXT_SETUP(cvlib_export_gem_pix);
//    FLEXT_SETUP(cvlib_copy);
    FLEXT_SETUP(cvlib_set_roi);

    FLEXT_SETUP(cvlib_canny);
   FLEXT_SETUP(cvlib_dilate);
   FLEXT_SETUP(cvlib_erode);
   FLEXT_SETUP(cvlib_median);
   FLEXT_SETUP(cvlib_gaussian);
    FLEXT_SETUP(cvlib_athreshold);
    FLEXT_SETUP(cvlib_find_contours);
    FLEXT_SETUP(cvlib_approx_poly);
    FLEXT_SETUP(cvlib_draw_contours);
//    FLEXT_SETUP(cvlib_convex_hull);
//    FLEXT_SETUP(cvlib_match_contours);
    FLEXT_SETUP(cvlib_hough_circles);
    FLEXT_SETUP(cvlib_circle);

    FLEXT_SETUP(cvlib_contours_segment_count);
    FLEXT_SETUP(cvlib_route_contours);
    FLEXT_SETUP(cvlib_contours_bounding_rect);
    FLEXT_SETUP(cvlib_contours_min_area_rect);
    FLEXT_SETUP (cvlib_contours_get);
    FLEXT_SETUP (cvlib_contours_hu_moments);
    FLEXT_SETUP (cvlib_contours_convexity2);/// to remove
    FLEXT_SETUP (cvlib_contours_convex_hull);
    FLEXT_SETUP (cvlib_contours_convexity_defects);
    FLEXT_SETUP (pix_opencv_contours_boundingrect);
    FLEXT_SETUP (pix_opencv_bgsubstract);
    FLEXT_SETUP (pix_opencv_hu_compare);
    FLEXT_SETUP (pix_opencv_threshold);
    FLEXT_SETUP (pix_opencv_contours_convexity);


    }

// setup the library (dll name, setup function name)
FLEXT_LIB_SETUP(cv,cvsetup)










