#include "cvlib_ImageFilter_1_1.h"





//****************************************************************
// cvlib_base methods

// *********** CONSTRUCTOR *****************

cvlib_ImageFilter_1_1::cvlib_ImageFilter_1_1()
{
	// define inlets:
	// first inlet must always by of type anything (or signal for dsp objects)
	AddInAnything();  // add one inlet for any message
    AddOutAnything();  // add one outlet for any message

    just_instantiated=1;

    cvImageSource=0;
    cvGemImage=0;
    cvImageDest=0;

    previousPixblock=0;

    pcvImageSource=0;

    cvImageSourceColorModel=0;

    cvImageSourceSID=-1;

   // allocate sid and register it for cvImageDest
    cvImageDestSID=cvlib_createHeader(CVLIB_TYPE_IPLIMAGE,CVLIB_STATUS_DATA_NOT_OWNED);



    output_dirty_flag=1;
    dest_width=0;
    dest_height=0;

    dest_channels=0;

        // filling tables of symbols with corresponding tag names
    cvlib_tagImage=gensym("cvImage");
    cvlib_tagGemState=gensym("gem_state");
    cvlib_tagContour=gensym("cvContour");



	// register methods
        FLEXT_ADDMETHOD_I(0,"cvImage",process_cvImage);  // register method for tag "cvImage" and int argument
        FLEXT_ADDBANG(0,reprocess_cvImage);
        FLEXT_ADDMETHOD_(0,"gem_state", process_gem_state);
}

// DESTRUCTOR

cvlib_ImageFilter_1_1::~cvlib_ImageFilter_1_1()
{
    if (cvImageDest)
        cvReleaseImage( &cvImageDest);
    cvlib_freeHeader(cvImageDestSID);
}







//  output any data stored in cvlib_pool, prepending sid by appropriate tag
void cvlib_ImageFilter_1_1::outputTaggedSID(int outlet, int sid)
{
       t_atom my_output;
    int status=cvlib_get_status(sid);
    if ((status!=CVLIB_STATUS_NULL)&&(status!=CVLIB_STATUS_ERROR))
    {
        int type=cvlib_get_type(sid);
        switch (type)
        {
            case CVLIB_TYPE_IPLIMAGE:
            {
                SETFLOAT(&my_output,sid);
                ToOutAnything(outlet,cvlib_tagImage,1,&my_output);
            }
            break;
            case CVLIB_TYPE_CONTOUR:
            {
                SETFLOAT(&my_output,sid);
                ToOutAnything(outlet,cvlib_tagContour,1,&my_output);
            }
            break;
            default: break;
        }
    }
}

// function called by process_cvImageTag
int cvlib_ImageFilter_1_1::check_roi_and_process_IplImage(IplImage * &Source, IplImage * &Dest)
{
  // post("check roi");
    int result=0;
    if (cvImageSource->roi!=0)
        result=process_roi_IplImage(Source,Dest);
    else
        result=process_IplImage(Source,Dest);
    return result;
}

int cvlib_ImageFilter_1_1::process_IplImage(IplImage * &Source, IplImage * &Dest)
{
/*
#define GL_RGBA 0x1908
#define GL_RGB 0x1907
#define GL_BGR 0x80E0
#define GL_BGRA 0x80E1
#define GL_RGB16 0x8054
#define GL_LUMINANCE 0x1909
#define GL_LUMINANCE_ALPHA 0x190A
#define GL_UNSIGNED_BYTE 0x1401
*/

/// TO DO : ADD YUV SUPPORT

    if (Source->colorModel[0]==(char)CVLIB_RGBA)
        return process_RGBA_image(Source,Dest);
    else if
        (Source->colorModel[0]==(char)CVLIB_BGRA)
        return process_BGRA_image(Source,Dest);
    else if
        (Source->colorModel[0]==(char)CVLIB_RGB)
        return process_RGB_image(Source,Dest);
    else if
        (Source->colorModel[0]==(char)CVLIB_BGR)
        return process_BGR_image(Source,Dest);
    else
    {
        switch (Source->nChannels)
        {
            case 4 : return process_4channels_image(Source,Dest);
                break;
            case 3 : return process_3channels_image(Source,Dest);
                break;
            case 2 : return process_2channels_image(Source,Dest);
                break;
            case 1 :
            {
                //post("one channel");
                return process_1channel_image(Source,Dest);
            }
                break;
            default :
            {
                cvlib_error("image with %d channels not supported", Source->nChannels);
                return 0;
            }
        }
    }

}

//virtual functions to be overrided by real class definition

int cvlib_ImageFilter_1_1::process_roi_IplImage(IplImage * &Source, IplImage * &Dest)
{
    return (process_IplImage(Source, Dest));
    /*cvlib_error("Region Of Interest is not supported");
    return 0;*/
}

int cvlib_ImageFilter_1_1::process_default_image(IplImage * &Source, IplImage * &Dest)
{
    cvlib_error("Image depth is not supported");
    return 0;
}

int cvlib_ImageFilter_1_1::process_RGBA_image(IplImage * &Source, IplImage * &Dest)
{
    return process_4channels_image(Source,Dest);
}

int cvlib_ImageFilter_1_1::process_BGRA_image(IplImage * &Source, IplImage * &Dest)
{
    return process_4channels_image(Source,Dest);
}

int cvlib_ImageFilter_1_1::process_4channels_image(IplImage * &Source, IplImage * &Dest)
{
    return process_default_image(Source,Dest);
}

int cvlib_ImageFilter_1_1::process_RGB_image(IplImage * &Source, IplImage * &Dest)
{
    return process_3channels_image(Source,Dest);
}

int cvlib_ImageFilter_1_1::process_BGR_image(IplImage * &Source, IplImage * &Dest)
{
    return process_3channels_image(Source,Dest);
}

int cvlib_ImageFilter_1_1::process_3channels_image(IplImage * &Source, IplImage * &Dest)
{
    return process_default_image(Source,Dest);
}

int cvlib_ImageFilter_1_1::process_2channels_image(IplImage * &Source, IplImage * &Dest)
{
    return process_default_image(Source,Dest);
}

int cvlib_ImageFilter_1_1::process_1channel_image(IplImage * &Source, IplImage * &Dest)
{
    return process_default_image(Source,Dest);
}


// functions attached to left inlet

 void cvlib_ImageFilter_1_1::process_cvImage(int sid)
{
    if (cvlib_isIplImage(sid))
    {
        // retrieve IplImage pointer from sid
        pcvImageSource=(IplImage **)cvlib_get_pdata(sid);

        // load new cvImageSource
        cvImageSource=(IplImage *)*pcvImageSource;
      //  post("lecturee cvimage nchannel=%d",cvImageSource->nChannels);
     //   post("lecturee cvimage colormodel=%c%c%c%c",cvImageSource->colorModel[0],cvImageSource->colorModel[1],cvImageSource->colorModel[2],cvImageSource->colorModel[3]);
        // do processing
        int result=check_roi_and_process_IplImage(cvImageSource,cvImageDest);
        if (result)
        {     // success : prepare output sid
            cvlib_set_pdata(cvImageDestSID,&cvImageDest);
            // for eventual gem export, update pixBlock data if necessary
            if (result & CVLIB_RESIZED)
            {
                output_pixBlock.image.data=(unsigned char *)cvImageDest->imageData;
                output_pixBlock.image.xsize = cvImageDest->width;
                output_pixBlock.image.ysize = cvImageDest->height;
                output_pixBlock.image.csize = cvImageDest->nChannels;
                output_pixBlock.image.type = GL_UNSIGNED_BYTE;
                output_pixBlock.image.format = cvlib_getIplImageColorspace(cvImageDest);
                output_pixBlock.image.notowned=1;
                output_pixBlock.image.upsidedown = /*1-*/cvImageDest->origin;
                cvImageDest->origin=cvImageSource->origin;
                    // invert image upside between opencv and gem
             //   output_pixBlock.image=output_imageStruct;
            }


            // set output dirty flag to 1 for gem_state export
            output_dirty_flag=1;
            // send dest image sid tag
            outputTaggedSID(0, cvImageDestSID);
        }
    }
    else
        cvlib_error("cvImage index %d doesn't correspond to an image", cvImageSourceSID);
}


void cvlib_ImageFilter_1_1::reprocess_cvImage()
{
    if (pcvImageSource!=0)
    {
        //Redirect cvImageSource pointer to original IplImage struct
        cvImageSource=(IplImage *)*pcvImageSource;
    }
    if (cvImageSource)
    {
        // an image is already loaded : do processing
        if (check_roi_and_process_IplImage(cvImageSource,cvImageDest))

        {
            cvImageDest->origin=cvImageSource->origin;
            // success : prepare output sid
            cvlib_set_pdata(cvImageDestSID,&cvImageDest);

            // send dest image sid tag
            outputTaggedSID(0, cvImageDestSID);


        }
    }
    else
        cvlib_error("no cvImage loadedcvlib source image not defined");
}


void cvlib_ImageFilter_1_1::process_gem_state(int argc, t_atom *argv)
{
    // gem_state tag recognized

    int result;
    input_image_type_has_changed=0;

    // check if format received is : gem_state (pointer) (pointer)
    if (argc==2)
    if (CanbePointer(argv[1]))
    {
        //can be pointer
        input_gem_state_pointer =  (GemState *) GetPointer(argv[1]);
        if ((!input_gem_state_pointer)|| (!input_gem_state_pointer->image))
        {
            //no pixblock !!
            input_image_type_has_changed = 1;  /// ?????
            input_image_data = 0;               /// ?????
        }
        else
        {       /// je ne comprends plus le &  !!!!
            if  (&input_gem_state_pointer->image->image == 0)
            {
                //we have a pixblock, but no image!
                input_image_type_has_changed = 1;   /// ?????
                input_image_data=0;                 /// ?????

            }
            else
            {
                 /// to do : check if input image has changed

                if (cvGemImage==0)
                {

                   // retrieve input image size/type and copy into IplImage cvImageSource
                    gemImageSourceWidth = input_gem_state_pointer->image->image.xsize;
                    gemImageSourceHeight = input_gem_state_pointer->image->image.ysize;
                    gemImageSourceChannels = input_gem_state_pointer->image->image.csize;

                  //    post(" create IplImage header");
                   if (cvGemImage==0)
                    cvImageSource=cvGemImage=cvCreateImageHeader(cvSize(gemImageSourceWidth,gemImageSourceHeight),
                                            IPL_DEPTH_8U,gemImageSourceChannels);
                }

                // reload source image if dirty flag is set
                if ((input_gem_state_pointer->image->newimage)||(input_gem_state_pointer->image!=previousPixblock)
                ||(input_gem_state_pointer->image->image.data!=previousPixblock->image.data))
                {
                     if (input_gem_state_pointer->image!=previousPixblock)
                     {
                       // post("image changes");
                        previousPixblock=input_gem_state_pointer->image;
                     }

                    // reload cvGemImage
                    cvImageSource=cvGemImage;
                    // disconnect cvlibImagesource pointer from any eventual cvImage IplImage
                    pcvImageSource=0;


                    // retrieve input image size/type and copy into IplImage cvImageSource
                    gemImageSourceWidth = cvImageSource->width = input_gem_state_pointer->image->image.xsize;
                    gemImageSourceHeight = cvImageSource->height = input_gem_state_pointer->image->image.ysize;

                    gemImageSourceChannels = cvImageSource->nChannels = input_gem_state_pointer->image->image.csize;
 /// UPDATE widthStep
                 //   post ("wstep=%d modulo=%d",gemImageSourceWidth * gemImageSourceChannels,4*int((gemImageSourceWidth * gemImageSourceChannels+3)/4));
                    cvImageSource->widthStep=4*int((gemImageSourceWidth * gemImageSourceChannels+3)/4);

                    cvImageSource->imageData = (char*)input_gem_state_pointer->image->image.data;
                    cvImageSource->imageSize = gemImageSourceWidth * gemImageSourceHeight * gemImageSourceChannels;
                    cvImageSource->origin=1-input_gem_state_pointer->image->image.upsidedown;

                    // encode colorspace into IplImage struct
                    cvlib_setIplImageColorspace(cvImageSource,input_gem_state_pointer->image->image.format);

                    // backup cvGemImage
                    cvGemImage=cvImageSource;

                    output_dirty_flag = 1;   // do processing
                }


        if (output_dirty_flag)
        {
            output_dirty_flag=0;
            if (cvImageSource)  //check if cvImageSource exists
                result = check_roi_and_process_IplImage(cvImageSource,cvImageDest);
            else
                result=CVLIB_FAILED;

            if (result)
            {
                // set dirty flag to 1
                output_pixBlock.newimage=1;

                if (result & CVLIB_RESIZED)
                {

            //    post ("widthStepsource=%d, dest=%d",cvImageSource->widthStep,cvImageDest->widthStep);
               // updating output pixBlock image parameters if dest image changed
                output_pixBlock.image.data=(unsigned char *)cvImageDest->imageData;
                output_pixBlock.image.xsize = cvImageDest->width;
                output_pixBlock.image.ysize = cvImageDest->height;
                output_pixBlock.image.csize = cvImageDest->nChannels;
                output_pixBlock.image.type = GL_UNSIGNED_BYTE;
                output_pixBlock.image.format = cvlib_getIplImageColorspace(cvImageDest);
                output_pixBlock.image.notowned=1;
                /// TO CHECK UPSIDEDOWN
                output_pixBlock.image.upsidedown = 1-cvImageSource->origin;
             //  post ("imagestruct w=%d h=%d c=%d", output_imageStruct.xsize,output_imageStruct.ysize,output_imageStruct.csize);
           //     output_pixBlock.image=output_imageStruct;
              //  post ("pixblock w=%d h=%d c=%d", output_pixBlock.image.xsize,output_pixBlock.image.ysize,output_pixBlock.image.csize);
                }
            }
        }
        // redirect input gemstate pointer pixblock to output pixblock
        input_gem_state_pointer->image=&output_pixBlock;
        input_gem_state_pointer->texCoords=0;
                    }
        }
        if (just_instantiated)
        {
            SETFLOAT(&flag,1);
            ToOutAnything(0, gensym("gem_state"),1,&flag);
            just_instantiated=0;
        }
        ToOutAnything(0,gensym("gem_state"),2,argv);

        // restore gemstate pointer
        /// TO DO : check if other things to restore
        input_gem_state_pointer->image=previousPixblock;
    }
    if (argc==1)
    {
        // reset gemimage pointer
        cvGemImage=0;
        ToOutAnything(0,gensym("gem_state"),1,argv);
        if(GetFloat(argv[0]))
            just_instantiated=0;
    }
}

