
#include "cvlib_athreshold.h"
// CONSTRUCTOR
cvlib_athreshold::cvlib_athreshold(int argc,t_atom *argv)
    {


        // add inlets for cv parameters
        AddInInt();   // maxvalue
        AddInInt();     // block size (default 3)


        // link inlets to corresponding methods
        FLEXT_ADDMETHOD_(0,"method",setMethod);
        FLEXT_ADDMETHOD_(0,"mode",setMode);
        FLEXT_ADDMETHOD_(0,"dim",setDim);
        FLEXT_ADDMETHOD(1,setMaxValue);
        FLEXT_ADDMETHOD(2,setBlockSize);



        // set default values
        threshold_max_value = 255;
        threshold_mode  = 0;
        threshold_method  = CV_ADAPTIVE_THRESH_MEAN_C;
        threshold_blocksize  = 3;
        threshold_dim = 0;

    }

// DESTRUCTOR
cvlib_athreshold::~cvlib_athreshold()
    {
// free extra ressources

    }

 void	    	setMode(int mode);
    void	    	setMethod(int method);
    void	    	setBlockSize(int blocksize);
    void	    	setDim(int dim);
    void            setMaxValue(int maxvalue);

void cvlib_athreshold::setMode(int mode)
{
    if ((mode==0 ) || ( mode==1 ) )
        threshold_mode = mode;
        // force image processing on next gemstate
    output_dirty_flag=1;
}
void cvlib_athreshold::setMethod(int method)
{
  if ( method==0 ) threshold_method = CV_ADAPTIVE_THRESH_MEAN_C;
  if ( method==1 ) threshold_method = CV_ADAPTIVE_THRESH_GAUSSIAN_C;

        // force image processing on next gemstate
    output_dirty_flag=1;
}
void cvlib_athreshold::setBlockSize(int blocksize)
{
    if (( blocksize>=3 ) && ( (blocksize+1)%2 == 0 ) )
  {
     threshold_blocksize = blocksize;
  }
        // force image processing on next gemstate
    output_dirty_flag=1;
}
void cvlib_athreshold::setDim(int dim)
{
    if ( dim>0 ) threshold_dim = dim;
        // force image processing on next gemstate
    output_dirty_flag=1;
}
void cvlib_athreshold::setMaxValue(int maxvalue)
{
    if (maxvalue>0 ) threshold_max_value = maxvalue;
        // force image processing on next gemstate
    output_dirty_flag=1;
}



//cvmedian works on any Channel images :
// only override process_default_image() function

int cvlib_athreshold::process_1channel_image(IplImage *&Source, IplImage * &Dest)
{
    //cvReleaseImage(&cvImageDest);
    int result=cvlib_prepareIplImage(&Dest, Source->width, Source->height, 1);
    // prepare dest image with same size as original, and 1 channel
    if (result)
    {
        // Applies fixed-level thresholding to single-channel array.
        switch(threshold_mode)
        {
        case 0:
           cvAdaptiveThreshold(Source, Dest, (float)threshold_max_value, threshold_method, CV_THRESH_BINARY, threshold_blocksize, threshold_dim);
           break;
        case 1:
           cvAdaptiveThreshold(Source, Dest, (float)threshold_max_value, threshold_method, CV_THRESH_BINARY_INV, threshold_blocksize, threshold_dim);
           break;
        }
        return result;
    }
    else
    {
        cvlib_error("error (re)creating target image");
        return CVLIB_FAILED;
    }
}

FLEXT_LIB_V("cvAthreshold pix_opencv_athreshold",cvlib_athreshold);


