////////////////////////////////////////////////////////
//
// GEM - Graphics Environment for Multimedia
//
// zmoelnig@iem.kug.ac.at
//
// Implementation file
//
//    Copyright (c) 1997-2000 Mark Danks.
//    Copyright (c) Gnther Geiger.
//    Copyright (c) 2001-2002 IOhannes m zmoelnig. forum::fr::umlute. IEM
//    Copyright (c) 2002 James Tittle & Chris Clepper
//    For information on usage and redistribution, and for a DISCLAIMER OF ALL
//    WARRANTIES, see the file, "GEM.LICENSE.TERMS" in this distribution.
//
/////////////////////////////////////////////////////////

#include "cvlib_contours_convexity2.h"
#include <stdio.h>

#define cvlib_contours_convexity2_extra_feature
/////////////////////////////////////////////////////////
//
// cvlib_contours_convexity2
//
/////////////////////////////////////////////////////////
// Constructor
//
/////////////////////////////////////////////////////////
cvlib_contours_convexity2 :: cvlib_contours_convexity2()
{
  AddInFloat();     //minarea
  AddInFloat();     //maxarea

  /////////////////////////////////////////////////////////
// static member function
//
/////////////////////////////////////////////////////////



  FLEXT_ADDMETHOD_F(0,gensym("minarea"),floatMinAreaMess);

  FLEXT_ADDMETHOD_F(0,gensym("maxarea"),floatMaxAreaMess);
#ifdef cvlib_contours_convexity2_extra_feature
    FLEXT_ADDMETHOD_I(0,gensym("margin"),setMargin);
//FLEXT_ADDMETHOD_I(0,"cvContours",process_cvContours);  // register method for tag "cvContours" and int argument





    AddOutList();    // deepest convexity default point
#endif
    AddOutInt();   // number convexity defects
    AddOutList();    // m_dataout

    minarea = 1;
  maxarea = 320*240;
  comp_xsize  = 0;
  comp_ysize  = 0;
  orig = NULL;
  gray = NULL;
  margin=15;

    stor01 = cvCreateMemStorage(0);
    stor02 = cvCreateMemStorage(0);
    stor03 = cvCreateMemStorage(0);
    stor04 = cvCreateMemStorage(0);
   stor05 = cvCreateMemStorage(0);

    defects=0;
    contours=0;
    convex_seq=0;
    approx_result=0;
    seqhull=0;

}

/////////////////////////////////////////////////////////
// Destructor
//
/////////////////////////////////////////////////////////
cvlib_contours_convexity2 :: ~cvlib_contours_convexity2()
{
    	//Destroy cv_images to clean memory
    cvReleaseMemStorage( &stor01 );
    cvReleaseMemStorage( &stor02 );
    cvReleaseMemStorage( &stor03 );
    cvReleaseMemStorage( &stor04 );
    cvReleaseMemStorage( &stor05 );
}



/////////////////////////////////////////////////////////
// processImage
//
/////////////////////////////////////////////////////////


int cvlib_contours_convexity2 :: process_1channel_image(IplImage * &Source, IplImage * &Dest)
{
    int result=cvlib_prepareIplImage(&Dest,Source->width,Source->height, 1);
    cvZero(Dest);

    int  count=0;
 //   CvSeq* approx_result;
    CvSeq* approximated_contours;
    CvSeq* approximated_contours_head;
    CvSeq* contours_head;

    int* hull;
    int hullsize;
    CvPoint* PointArray;
    CvPoint* ConvexPointArray;
    CvConvexityDefect* defectArray;

// clear mem storage
    if(contours!=0)
        {
            contours=0;
            cvClearMemStorage(stor01);
        }
    if(approx_result!=0)
        {
            approx_result=0;
            cvClearMemStorage(stor02);
        }

    if(defects!=0)
        {
            defects=0;
            cvClearMemStorage(stor03);
        }

    if(convex_seq!=0)
        {
            convex_seq=0;
            cvClearMemStorage(stor04);
        }
    if(seqhull!=0)
        {
            seqhull=0;
            cvClearMemStorage(stor05);
        }




#ifdef cvlib_contours_convexity2_extra_feature

            float test=-1;
            CvPoint2D32f depth_point32F;
            /// find deepest convexity default point
            int deepest_x=0;
            int deepest_y=0;
            float deepest_distance=-1;
            float depth_distance=0;
#endif


    cvFindContours( Source, stor01, &contours, sizeof(CvContour), CV_RETR_EXTERNAL, CV_CHAIN_APPROX_SIMPLE, cvPoint(0,0) );
    //  if (contours) contours = cvApproxPoly( contours, sizeof(CvContour), stor02, CV_POLY_APPROX_DP, cvContourPerimeter(contours)*0.001,0 );
    //  if (contours) contours = cvApproxPoly( contours, sizeof(CvContour), stor02, CV_POLY_APPROX_DP, 3, 1 );
    contours_head=contours;
    // Approximation itrative de tous les contours
    approximated_contours=approximated_contours_head=0;


    while (contours)
    {
        count++;
        approx_result = cvApproxPoly( contours, sizeof(CvContour), stor02,
                   CV_POLY_APPROX_DP, cvContourPerimeter(contours)*0.004, 0 );

        // copy CV_SEQ_FLAG_HOLE
        approx_result->flags|=contours->flags & CV_SEQ_FLAG_HOLE;

        // store result in approximated_contours sequence
        if (count==1)
        {
            // first occurence : store in head of dynamic structure
            approximated_contours_head=approx_result;
            approximated_contours=approx_result;
        }
        else
        {
            // link to dynamic structure
            approximated_contours->h_next=approx_result;
            // change current poointer
            approximated_contours=approx_result;
        }
        // let's see next contour
        contours = contours->h_next;
    }   // end of while loop

    // go to first approximated contour
    contours=approximated_contours_head;

    int i = 0;
    int area = 0;
    int selected = -1;

    //busquem el contorn mes gran - look at biggest contour('s bounding rect)

    CvSeq* biggest_contour=0;
    for( ; contours != 0; contours = contours->h_next )
    {
        CvRect rect;
        rect = cvContourBoundingRect(contours, 1);
        if  ( (rect.width*rect.height) > area )
		{
            biggest_contour=contours;
            selected = i;

            area = rect.width*rect.height;
		}
        i++;
	}
    contours=biggest_contour;
    if(contours)
    {
        count=contours->total;
       // post ("count=%d", count);
        // Alloc memory for contour point set.

        PointArray = (CvPoint*)malloc(count * sizeof(CvPoint) );

        // Alloc memory for indices of convex hull vertices.
        hull = (int*)malloc(sizeof(int) * count);

        // Get contour point set.
        cvCvtSeqToArray(biggest_contour, PointArray, CV_WHOLE_SEQ);

        // Find convex hull for curent contour.
        cvConvexHull( PointArray,
                      count,
                      NULL,
                      CV_COUNTER_CLOCKWISE,
                      hull,
                      &hullsize);

        // Find convex hull for current contour.
        // This required for cvConvexityDefects().
       seqhull = cvConvexHull2( contours,stor05,
                             CV_COUNTER_CLOCKWISE,
                            0 );

     //   post("hullsize=%d, seqhull->total=%d",hullsize, seqhull->total);/// the same

//#ifdef cvlib_contours_convexity2_extra_feature
       // creating cvseq contour from convex hull for further purposes (cvPointPolygonTest)
        ConvexPointArray = (CvPoint*)malloc(sizeof(CvPoint)*hullsize);
        CvSeqWriter writer;
        cvStartWriteSeq( CV_SEQ_ELTYPE_POINT|CV_SEQ_KIND_CURVE|CV_SEQ_FLAG_CLOSED, sizeof(CvSeq), sizeof(CvPoint), stor04, &writer );
        for( int ii=0;ii<hullsize; ii++ )
        {
            CV_WRITE_SEQ_ELEM( PointArray[hull[ii]], writer );
        }
        convex_seq = cvEndWriteSeq( &writer );
        // draw convex contour
        cvDrawContours( Dest, convex_seq, CV_RGB(200,200,200), CV_RGB(200,200,200), 2,2,  8 );
//#endif
      //  post("hullsize=%d", hullsize);
        // This required for cvConvexityDefects().
        // Otherwise cvConvexityDefects() falled.

        if( hullsize >= 4 )   ///  >=3 ???
    {
        // Find defects of convexity of current contours.
        defects = cvConvexityDefects( contours,
                            seqhull,
                            stor03);

        int nomdef = defects->total; // defect amount
     //   post("defects->total=%d",defects->total);
        // Alloc memory for defect set.
        defectArray = (CvConvexityDefect*)malloc(sizeof(CvConvexityDefect)*nomdef);
        if(nomdef > 0)
        {
            // Get defect set.
            cvCvtSeqToArray(defects,defectArray, CV_WHOLE_SEQ);

            // Draw marks for all defects.
            for(i=0; i<nomdef; i++)
            {
#ifdef cvlib_contours_convexity2_extra_feature

               // do polygon test to  see if depth_point is not part of contour
                depth_point32F.x=defectArray[i].depth_point->x;
                depth_point32F.y=defectArray[i].depth_point->y;
                test=cvPointPolygonTest(convex_seq,depth_point32F,1);

                depth_distance= (defectArray[i].start->x-defectArray[i].depth_point->x)*(defectArray[i].start->x-defectArray[i].depth_point->x)
                                +
                                (defectArray[i].start->y-defectArray[i].depth_point->y)*(defectArray[i].start->y-defectArray[i].depth_point->y)
                                +
                                (defectArray[i].end->x-defectArray[i].depth_point->x)*(defectArray[i].end->x-defectArray[i].depth_point->x)
                                +
                                (defectArray[i].end->y-defectArray[i].depth_point->y)*(defectArray[i].end->y-defectArray[i].depth_point->y);
                if ((test>margin)&&(depth_distance > deepest_distance))
                {
                        deepest_distance=depth_distance;
                        deepest_x=defectArray[i].depth_point->x;
                        deepest_y=defectArray[i].depth_point->y;
                }
#endif

                cvLine(Dest, *(defectArray[i].start), *(defectArray[i].depth_point),CV_RGB(0,0,255),1, CV_AA, 0 );
                cvCircle( Dest, *(defectArray[i].depth_point), 5, CV_RGB(128,128,128), -1, 8,0);
                cvCircle( Dest, *(defectArray[i].start), 5, CV_RGB(128,128,128), -1, 8,0);
                cvLine(Dest, *(defectArray[i].depth_point), *(defectArray[i].end),CV_RGB(128,128,128),1, CV_AA, 0 );

                // output list of defects points (start, depth, end)
                t_atom rlist[7];
                SETFLOAT(&rlist[0], i);
                SETFLOAT(&rlist[1], defectArray[i].start->x);
                SETFLOAT(&rlist[2], defectArray[i].start->y);
                SETFLOAT(&rlist[3], defectArray[i].depth_point->x);
                SETFLOAT(&rlist[4], defectArray[i].depth_point->y);
                SETFLOAT(&rlist[5], defectArray[i].end->x);
                SETFLOAT(&rlist[6], defectArray[i].end->y);
                #ifdef cvlib_contours_convexity2_extra_feature
                ToOutList(3,  7, rlist );
                #else
                ToOutList(2,  7, rlist );
                #endif
            }
        }
        // output number of convexity defects
        #ifdef cvlib_contours_convexity2_extra_feature
        ToOutFloat( 2, nomdef );
        #else
        ToOutFloat( 1, nomdef );
        #endif

        // output biggest convexity defect
#ifdef cvlib_contours_convexity2_extra_feature
        if (deepest_distance >=0)
        {
            t_atom rlist[3];
            SETFLOAT(&rlist[0], deepest_x);
            SETFLOAT(&rlist[1], deepest_y);
            SETFLOAT(&rlist[2], test);
            cvCircle( Dest, cvPoint(deepest_x,deepest_y), 11, CV_RGB(255, 255, 255), -1, 8,0);
            ToOutList(1, 3, rlist);
        }
#endif
	   //j++;

        // Free memory.
        free(defectArray);
    }

        // Draw current contour.

    cvDrawContours( Dest, contours, CV_RGB(200,200,200), CV_RGB(200,200,200), -1,1,8  );



        // Free memory.
        free(PointArray);
        free(ConvexPointArray);
        free(hull);

    }



    //if (defects) cvClearSeq(defects);
    //if (seqhull) cvClearSeq(seqhull);



    return result;
}

// 	virtual void 	processRGBAImage(imageStruct &image);
int	cvlib_contours_convexity2 ::process_RGBA_image(IplImage * &Source, IplImage * &Dest)
{
    // convert image in gray and call process_1channelImage
    int result=cvlib_prepareIplImage(&grayIn, Source->width, Source->height,1);
    cvlib_prepareIplImage(&Dest, Source->width, Source->height,4);
    cvCvtColor(Source, grayIn, CV_BGRA2GRAY);
    process_1channel_image(grayIn, grayOut);
    cvCvtColor(grayOut, Dest, CV_GRAY2BGRA);
    return (result);
}


// 	virtual void 	processRGBImage(imageStruct &image);
int cvlib_contours_convexity2 ::process_RGB_image(IplImage * &Source, IplImage * &Dest)
{
    // convert image in gray and call process_1channelImage
    int result=cvlib_prepareIplImage(&grayIn, Source->width, Source->height,1);
    cvlib_prepareIplImage(&Dest, Source->width, Source->height,3);
    cvCvtColor(Source, grayIn, CV_BGR2GRAY);
    process_1channel_image(grayIn, grayOut);
    cvCvtColor(grayOut, Dest, CV_GRAY2BGR);
    return (result);
}





/////////////////////////////////////////////////////////
// floatThreshMess
//
/////////////////////////////////////////////////////////
void cvlib_contours_convexity2 :: floatMinAreaMess (float minarea)
{
  if (minarea>0) this->minarea = (int)minarea;
      // force image processing on next gemstate
    output_dirty_flag=1;
}

void cvlib_contours_convexity2 :: floatMaxAreaMess (float maxarea)
{
  if (maxarea>0) this->maxarea = (int)maxarea;
      // force image processing on next gemstate
    output_dirty_flag=1;
}
void cvlib_contours_convexity2 :: setMargin (int i)
{
  margin=i;
      // force image processing on next gemstate
    output_dirty_flag=1;
}




FLEXT_LIB("cvContoursConvexity2",cvlib_contours_convexity2);
