
#include "cvlib_contours_get.h"
#define CVLIB_DEBUG post


// CONSTRUCTOR
cvlib_contours_get::cvlib_contours_get(int argc,t_atom *argv)
    {
       t_symbol * parameter;
        nb_of_features=min(argc,10);
        if (nb_of_features==0)
        {
            post("cvContoursFeature : please specify at least one feature : Area BoundingRect Flags HuMoments LogHuMoments IsConvex MinAreaRect Perimeter SegmentCount");
            InitProblem();
        }

        AddInAnything();  // cvContours inlet

        // add outlets for every method invoqued as arguments
        // make each pointer in array point to corresponding feature extraction functions
        for (int i=0;i<nb_of_features;i++)
        {
            AddOutAnything();
            funcArr[i]=&cvlib_contours_get::get_Nothing;
            if (IsSymbol(argv[i]))
            {
                parameter=(t_symbol *)GetSymbol(argv[i]);

                if (parameter==gensym("Area"))
                {
                    funcArr[i]=&cvlib_contours_get::get_Area;

                    continue;
                }
                if (parameter==gensym("BoundingRect"))
                {
                    funcArr[i]=&cvlib_contours_get::get_BoundingRect;
                    continue;
                }
 /*                if (parameter==gensym("Centroids"))
                {
                    //post ("outlet %d:Centroids",i+1);
 //                  funcArr[i]=&cvlib_contours_get::get_MinAreaRect;
                    continue;
                }*/
                 if (parameter==gensym("DominantPoints"))
                {
                    //post ("outlet %d:DominantPoints",i+1);
 //                  funcArr[i]=&cvlib_contours_get::get_MinAreaRect;
                    continue;
                }
                if (parameter==gensym("FitEllipse"))
                {
                    //post ("outlet %d:FitEllipse",i+1);
 //                  funcArr[i]=&cvlib_contours_get::get_MinAreaRect;
                    continue;
                }
                if (parameter==gensym("Flags"))
                {
                    //post ("outlet %d:Flag",i+1);
 //                  funcArr[i]=&cvlib_contours_get::get_MinAreaRect;
                    continue;
                }
                if (parameter==gensym("HuMoments"))
                {
                    funcArr[i]=&cvlib_contours_get::get_HuMoments;
                    continue;
                }
                if (parameter==gensym("LogHuMoments"))
                {
                   funcArr[i]=&cvlib_contours_get::get_LogHuMoments;
                    continue;
                }
                if (parameter==gensym("IsConvex"))
                {
                    funcArr[i]=&cvlib_contours_get::get_IsConvex;
                    continue;
                }
                if (parameter==gensym("MinAreaRect"))
                {
                   funcArr[i]=&cvlib_contours_get::get_MinAreaRect;
                    continue;
                }
                if (parameter==gensym("Perimeter"))
                {
                    funcArr[i]=&cvlib_contours_get::get_Perimeter;
                    continue;
                }
                if (parameter==gensym("SegmentCount"))
                {
                    funcArr[i]=&cvlib_contours_get::get_SegmentCount;
                    continue;
                }
                post("error: %s is not a feature",GetString(argv[i]));
                InitProblem();
            }
            else
            {
                post("error: incorrect parameter type (float)");
                InitProblem();
            }
        }

        FLEXT_ADDMETHOD_I(0,"cvContours",process_cvContours);  // register method for tag "cvContours" and int argument
}

// DESTRUCTOR
cvlib_contours_get::~cvlib_contours_get()
    {

    }


void cvlib_contours_get::process_cvContours(int i)
{
   if (cvlib_get_type(i)==CVLIB_TYPE_CONTOUR)
    {
        contours=(CvSeq*)cvlib_get_pdata(i);
        int  count=0;
        while (contours)
        {
            for(int k=nb_of_features-1;k>=0;k--)
            {
                // appel du pointeur de fonction
                (*this.*funcArr[k])(contours,k);
            }

            count++;
            contours = contours->h_next;
            if (count > CVLIB_MAX_CONTOURS)
                continue;
        }
    }
}



// feature extraction functions

void cvlib_contours_get::get_Area(CvSeq* contour,int feature_outlet)
{
    ToOutFloat(feature_outlet, fabs(cvContourArea( contour, CV_WHOLE_SEQ)) );
}

void cvlib_contours_get::get_BoundingRect(CvSeq* contour,int feature_outlet)
{
    rect = cvContourBoundingRect(contour, 1);
    SETFLOAT(&result[0],rect.x);
    SETFLOAT(&result[1],rect.y);
    SETFLOAT(&result[2],rect.width);
    SETFLOAT(&result[3],rect.height);

    ToOutList(feature_outlet,4, result);
}

/*void cvlib_contours_get::get_Centroids(CvSeq* contour,int feature_outlet)
{
}*/

void cvlib_contours_get::get_DominantPoints(CvSeq* contour,int feature_outlet)
{
}

void cvlib_contours_get::get_FitEllipse(CvSeq* contour,int feature_outlet)
{
}

void cvlib_contours_get::get_Flags(CvSeq* contour,int feature_outlet)
{
}

void cvlib_contours_get::get_HuMoments(CvSeq* contour,int feature_outlet)
{
            cvContourMoments(contours,&moments);
            cvGetHuMoments(&moments,&HuMoments);

            moment_value[0]=HuMoments.hu1;
            moment_value[1]=HuMoments.hu2;
            moment_value[2]=HuMoments.hu3;
            moment_value[3]=HuMoments.hu4;
            moment_value[4]=HuMoments.hu5;
            moment_value[5]=HuMoments.hu6;
            moment_value[6]=HuMoments.hu7;

            for (int k=0;k<7;k++)
                SETFLOAT(&result[k],moment_value[k]);

            ToOutList(feature_outlet,7, result);
}
void cvlib_contours_get::get_LogHuMoments(CvSeq* contour,int feature_outlet)
{
            float temp;
            cvContourMoments(contours,&moments);
            cvGetHuMoments(&moments,&HuMoments);


            moment_value[0]=HuMoments.hu1;
            moment_value[1]=HuMoments.hu2;
            moment_value[2]=HuMoments.hu3;
            moment_value[3]=HuMoments.hu4;
            moment_value[4]=HuMoments.hu5;
            moment_value[5]=HuMoments.hu6;
            moment_value[6]=HuMoments.hu7;

// convert into log values
            for (int k=0;k<7;k++)
                {
                    temp=moment_value[k];
                    moment_value[k]=((2*(temp>0)-1))*log(abs(temp));
                    SETFLOAT(&result[k],moment_value[k]);
                }
            ToOutList(feature_outlet,7, result);

}
void cvlib_contours_get::get_IsConvex(CvSeq* contour,int feature_outlet)
{
    if(cvCheckContourConvexity(contour))
        ToOutFloat (feature_outlet,1);
    else
        ToOutFloat (feature_outlet,0);
}

void cvlib_contours_get::get_MinAreaRect(CvSeq* contour,int feature_outlet)
{
    min_area_rect = cvMinAreaRect2( contour, 0);
    SETFLOAT(&result[0],min_area_rect.center.x);
    SETFLOAT(&result[1],min_area_rect.center.y);
    SETFLOAT(&result[2],min_area_rect.size.width);
    SETFLOAT(&result[3],min_area_rect.size.height);
    SETFLOAT(&result[4],min_area_rect.angle);
    ToOutList(feature_outlet,5, result);
}

void cvlib_contours_get::get_Nothing(CvSeq* contour,int feature_outlet)
{
    // blank function
}

void cvlib_contours_get::get_Perimeter(CvSeq* contour,int feature_outlet)
{
    ToOutFloat(feature_outlet, cvArcLength( contour, CV_WHOLE_SEQ, 1 ));


}

void cvlib_contours_get::get_SegmentCount(CvSeq* contour,int feature_outlet)
{
    ToOutFloat(feature_outlet, contour->total);
}


FLEXT_LIB_V("cvContoursFeature",cvlib_contours_get);
