
#include "cvlib_hough_circles.h"
#define CVLIB_DEBUG post


// CONSTRUCTOR
cvlib_hough_circles::cvlib_hough_circles(int argc,t_atom *argv)
    {
        AddInAnything();
        AddOutAnything();

        // register settings methods
        FLEXT_ADDMETHOD_I(1,"min_distance", set_min_distance);
        FLEXT_ADDMETHOD_I(1,"canny", set_canny_threshold);
        FLEXT_ADDMETHOD_I(1,"accumulator", set_accumulator_threshold);
        FLEXT_ADDMETHOD_I(1,"min_radius", set_min_radius);
        FLEXT_ADDMETHOD_I(1,"max_radius", set_max_radius);
        FLEXT_ADDMETHOD_I(1,"dp", set_dp);
        FLEXT_ADDMETHOD_I(1,"nightmode", set_nightmode);

  ///      FLEXT_ADDMETHOD_I(1,"draw", draw);

        // create storage
        storage = cvCreateMemStorage(0);
        // create contour
        cvSeqSID= cvlib_createHeader(CVLIB_TYPE_CONTOUR,CVLIB_STATUS_EXTERNAL_POINTER);

        //  init parameters default values

        accumulator_threshold=100;  // default value
        canny_threshold=100;
        min_distance=20;
        min_radius=20;
        max_radius=50;
        dp=2;
        nightmode=0;


            // argument extraction
            /*   if(argc >= 1)
                if  (IsFloat(argv[0]))
                set_iterations(GetInt(argv[0]));  */
    }

// DESTRUCTOR
cvlib_hough_circles::~cvlib_hough_circles()
    {
        cvlib_freeHeader(cvSeqSID);

    // free storage
        cvReleaseMemStorage(&storage);
    }


/*void cvlib_hough_circles::bang()
{
        send_cvContourSID();
}
*/
void cvlib_hough_circles::set_accumulator_threshold(int i)
{
    accumulator_threshold=min(400,max(1, i));
        // force image processing on next gemstate
    output_dirty_flag=1;
}

void cvlib_hough_circles::set_canny_threshold(int i)
{
    canny_threshold=min(400,max(1, i));
        // force image processing on next gemstate
    output_dirty_flag=1;
}

void cvlib_hough_circles::set_nightmode(int i)
{
    nightmode=i;
        // force image processing on next gemstate
    output_dirty_flag=1;
}

void cvlib_hough_circles::set_dp(int i)
{
    dp=max(1, i);
        // force image processing on next gemstate
    output_dirty_flag=1;
}

void cvlib_hough_circles::set_min_distance(int i)
{
    min_distance=min(400,max(1, i));
        // force image processing on next gemstate
    output_dirty_flag=1;
}

void cvlib_hough_circles::set_min_radius(int i)
{
    min_radius=min(400,max(1, i));
        // force image processing on next gemstate
    output_dirty_flag=1;
}

void cvlib_hough_circles::set_max_radius(int i)
{
    max_radius=min(400,max(1, i));
        // force image processing on next gemstate
    output_dirty_flag=1;
}

int cvlib_hough_circles::process_1channel_image(IplImage * &Source, IplImage * &Dest)
{

    int result=cvlib_prepareIplImage(&Dest, Source->width, Source->height,1);
if (!nightmode)
   cvCopyImage(Source, Dest);
 else
   cvZero(Dest);



    circles = cvHoughCircles( Source, storage, CV_HOUGH_GRADIENT, dp, min_distance, canny_threshold, accumulator_threshold, min_radius , max_radius );
    current_circle = circles;
   //         post ("found_circles = %d", circles->total);

   /// Add flag to process drawing or not
    int i;
    for( i = 0; i < circles->total; i++ )
    {
        float* p = (float*)cvGetSeqElem( circles, i );
      //  cvCircle( Dest, cvPoint(cvRound(p[0]),cvRound(p[1])), 3, CV_RGB(255,255,255), -1, 8, 0 );
        cvCircle( Dest, cvPoint(cvRound(p[0]+4),cvRound(p[1]+4)), cvRound(p[2]+4), CV_RGB(255,255,255), -1, 8, 0 );
    }

    // set instance pointer to contours
    cvlib_set_pdata(cvSeqSID,circles);

    // send cv contour SID

    SETFLOAT(&my_output,cvSeqSID);

        /// Is "cvSeq" a good tag choice ?
    ToOutAnything(1,gensym("cvSeq"),1,&my_output);


    // clear memory storage - reset free space position
    cvClearMemStorage(storage);

return result;
}


FLEXT_LIB_V("cvHoughCircles",cvlib_hough_circles);




