
#include "cvlib_route_contours.h"
#define CVLIB_DEBUG post


// CONSTRUCTOR
cvlib_route_contours::cvlib_route_contours(int argc,t_atom *argv)
{

        parameters_count=0;
 // argument extraction
        int ok=0;
        if((argc >= 1)&&(argc<=max_route_criteria))
        {
            ok=1;
            parameters_count=argc;
            for(int i= 0; i<parameters_count;i++)
            {
                if  (!IsFloat(argv[i]))
                {
                    ok=0;
                    break;
                }
                else
                    matching_values[i]=GetInt(argv[i]);

            }
        }
        if (!ok)
            InitProblem(); //Signal a construction problem
        else
        {
            AddInAnything();        // cvContours messages
            AddInAnything();        // matrix messages

            for(int i=0;i<parameters_count;i++)     // create as much outlets as arguments
            {
                AddOutAnything();
                cvSeqSID[i]= cvlib_createHeader(CVLIB_TYPE_CONTOUR,CVLIB_STATUS_EXTERNAL_POINTER);
            }
            // add one outlet for unmatched contours
            AddOutAnything();

            // register methods
            FLEXT_ADDMETHOD_I(0,"cvContours",process_cvContours);  // register method for tag "cvContours" and int argument
            FLEXT_ADDMETHOD(1,process_list);
            FLEXT_ADDMETHOD_II(1,"set", set_routing_element);

            ///obsolete
            FLEXT_ADDMETHOD_(1,gensym("matrix"),process_matrix);

            // init flags
            routing_vector_size=0;
            contour_processed=0;        // flag for resetting routing vector

            // create storage
            storage1 = cvCreateMemStorage(0);

            // create contour sid for rejected contours
            cvSeqSID_right= cvlib_createHeader(CVLIB_TYPE_CONTOUR,CVLIB_STATUS_EXTERNAL_POINTER);
    }
}

// DESTRUCTOR
cvlib_route_contours::~cvlib_route_contours()
    {

        cvlib_freeHeader(cvSeqSID_right);

        for(int i=0;i<parameters_count;i++)
            cvlib_freeHeader(cvSeqSID[i]);

     // free storage
        cvReleaseMemStorage(&storage1);
    }

/// TO DO : add method for ramdom access


void cvlib_route_contours::process_matrix(int argc,t_atom *argv)
{
    routing_vector_size=0;
    contour_processed=0;
    if (argc>=2)
        if ((IsFloat(argv[0]))&&IsFloat(argv[1]))

            if (GetFloat(argv[1])==1)
            {
                routing_vector_size=GetInt(argv[0]);
                if(routing_vector_size!=argc-2)
                {
                    cvlib_error("cvRouteContours error : incorrect vector length");
                    routing_vector_size=0;
                }
            }

            else
                cvlib_error("cvRouteContours error : incorrect matrix dimensions [%d %d]",(int)GetFloat(argv[0]),(int)GetFloat(argv[1]));
        else
            cvlib_error("cvRouteContours error : incorrect matrix arguments type");
    if (routing_vector_size)
    {

        for (int i=0;i<routing_vector_size; i++)
            if (IsFloat(argv[i+2]))
                routing_vector[i]=GetInt(argv[i+2]);
            else
                {
                    cvlib_error("cvRouteContours error : incorrect vector element type");
                    routing_vector_size=0;
                    break;
                }
    }

 }
void cvlib_route_contours::set_routing_element(int index, int value)
{
    if ((index<CVLIB_MAX_CONTOURS)&&(index>=0))
    {
        routing_vector_size=max(routing_vector_size, index+1);
        routing_vector[index]=value;
    }
    else
        cvlib_error("cvRouteContours error : specified index (%d) out of range", index);
}
void cvlib_route_contours::process_list(int argc,t_atom *argv)
{
    if (argc==0)   // bang : reset routing vector
    {
        for (int k=0; k<CVLIB_MAX_CONTOURS;k++)
            routing_vector[k]=0;
        routing_vector_size=0;

    }
            //routing_vector_size+=argc;
    if (argc)
    {
        if (contour_processed)  // reset routing vector
        {
            routing_vector_size=0;
            contour_processed=0;
        }
        if(argc+routing_vector_size<CVLIB_MAX_CONTOURS)
        {
            for (int i=0;i<argc; i++)
            {
                if (IsFloat(argv[i]))
                    routing_vector[routing_vector_size+i]=GetInt(argv[i]);
                else
                    {
                        cvlib_error("cvRouteContours error : incorrect vector element type");

                        break;
                    }
            }
            routing_vector_size+=argc;
        }
        else
            cvlib_error("cvRouteContours error : max vector length overriden");



    }
 }




void cvlib_route_contours::send_cvContourSID(int outlet, int sid)
{
    if (sid>=0)
        {
        SETFLOAT(&my_output,sid);
        /// Is "cvContours" a good tag choice ?
         ToOutAnything(outlet,gensym("cvContours"),1,&my_output);
        }
}



void cvlib_route_contours::process_cvContours(int sid)
{

    if (cvlib_get_type(sid)==CVLIB_TYPE_CONTOUR)
    {
        current_contour=(CvSeq*)cvlib_get_pdata(sid);

        // init values
        for(int i=0;i<parameters_count;i++)
        {
            contour_head[i]=0;
            contour_current[i]=0;
        }
        head_contour_right=0;
        current_contour_right=0;

        int index=0;
        while (current_contour)
        {
            int matched=-1;
            //look if routing vector element matches with one of the arguments
            if (index< routing_vector_size)
            {
                for(int k=0;k<parameters_count;k++)
                {
                    if(routing_vector[index]==matching_values[k])
                    {
                        matched=k;
                        break;
                    }
                }
            }
            if(matched==-1)
            // push contour into rightmost sequence
            {
                if(head_contour_right==0)
                    current_contour_right=head_contour_right=(CvSeq*)cvCloneSeq(current_contour,storage1);
                else
                {
                    current_contour_right->h_next=(CvSeq*)cvCloneSeq(current_contour,storage1);
                    current_contour_right=current_contour_right->h_next;
                }
            }
            else
                // push contour into  sequence [matched]
            {
                if(contour_head[matched]==0)
                        contour_current[matched]=contour_head[matched]=(CvSeq*)cvCloneSeq(current_contour,storage1);
                else
                {
                    contour_current[matched]->h_next=(CvSeq*)cvCloneSeq(current_contour,storage1);
                    contour_current[matched]=contour_current[matched]->h_next;
                }
            }
            // go to next contour
            current_contour=current_contour->h_next;
            index++;
        }
            // close properly cvsequences
            for(int k=0;k<parameters_count;k++)
            {
                if(contour_current[k]!=0)
                contour_current[k]->h_next=0;
            }
            // close properly rightmost cvsequence
            if(current_contour_right!=0)
                current_contour_right->h_next=0;

            // set instance pointer to contours
            for(int k=0;k<parameters_count;k++)
            {
                cvlib_set_pdata(cvSeqSID[k],contour_head[k]);

            }
            cvlib_set_pdata(cvSeqSID_right,head_contour_right);

            // send cv contour SIDs
            send_cvContourSID(parameters_count,cvSeqSID_right);

            for(int k=parameters_count-1;k>=0;k--)
            {
                send_cvContourSID(k,cvSeqSID[k]);

            // clear memory storage - reset free space position
            cvClearMemStorage(storage1);
        }
    contour_processed=1;
    }

}


FLEXT_LIB_V("cvRouteContours",cvlib_route_contours);




