#ifndef INCLUDE_gem_headers_H_
#define INCLUDE_gem_headers_H_

//----------------------------------------------
//----------------------------------------------
//                   gem_headers
//   trash-import of some gem_headers
//   (header definitions extracted from gem headers)
//    glew.h ,  GemPixUtil.h, GemState.h
//
/// to do : properly import true GEM headers
//
//------------------------------------------------

// frome Base/glew.h
#define GL_RGBA 0x1908
#define GL_RGB 0x1907
#define GL_BGR 0x80E0
#define GL_BGRA 0x80E1
#define GL_RGB16 0x8054
#define GL_LUMINANCE 0x1909
#define GL_LUMINANCE_ALPHA 0x190A
#define GL_UNSIGNED_BYTE 0x1401

#define GEM_EXTERN

//from glew.h
#if defined(__APPLE__)
typedef unsigned long GLenum;
typedef unsigned long GLbitfield;
typedef unsigned long GLuint;
typedef long GLint;
typedef long GLsizei;
#else
typedef unsigned int GLenum;
typedef unsigned int GLbitfield;
typedef unsigned int GLuint;
typedef int GLint;
typedef int GLsizei;
#endif
typedef unsigned char GLboolean;
typedef signed char GLbyte;
typedef short GLshort;
typedef unsigned char GLubyte;
typedef unsigned short GLushort;
typedef unsigned long GLulong;
typedef float GLfloat;
typedef float GLclampf;
typedef double GLdouble;
typedef double GLclampd;
typedef void GLvoid;
#if defined(_MSC_VER) && _MSC_VER < 1400
typedef __int64 GLint64EXT;
typedef unsigned __int64 GLuint64EXT;
#else
typedef signed long long GLint64EXT;
typedef unsigned long long GLuint64EXT;
#endif





// from GemPixUtil.h
struct GEM_EXTERN imageStruct
{
  //imageStruct();
  //~imageStruct();

  void info();
  //////////
  // columns
  unsigned char* allocate(size_t size);
  unsigned char* allocate();

  // if we have allocated some space already, only re-allocate when needed.
  unsigned char* reallocate(size_t size);
  unsigned char* reallocate();

  // delete the buffer (if it is ours)
  void clear();


  //////////
  // dimensions of the image
  GLint         xsize;
  GLint   	    ysize;

  //////////
  // (average) width of 1 pixel (LUMINANCE = 1, RGBA = 4, YUV = 2)
  GLint   	    csize;

  //////////
  // data type - always GL_UNSIGNED_BYTE (except for OS X)
  GLenum          type;

  //////////
  // the format - either GL_RGBA, GL_LUMINANCE
  // or GL_YCBCR_422_GEM (which is on mac-computers GL_YCBCR_422_APPLE)
  GLenum          format;

  //////////
  // is this owned by us (? what about underscores ?)
  int notowned;

  //////////
  // gets a pixel
  /* X,Y are the coordinates
   * C is the offset in the interleaved data (like chRed==0 for red)
   * you should use chRed instead of 0 (because it might not be 0)
   */
  // heck, why are X&Y swapped ?? (JMZ)
  inline unsigned char GetPixel(int Y, int X, int C)
  { return(data[Y * xsize * csize + X * csize + C]); }

  //////////
  // sets a pixel
  /* while X and Y should be clear (coordinates),
   * C is the offset (like chRed==0 for red).
   * VAL is the value to set.
   */
  inline void SetPixel(int Y, int X, int C, unsigned char VAL)
  { data[Y * xsize * csize + X * csize + C] = VAL; }


  /////////
  // gets the color of a pixel
  void getRGB(int X, int Y, unsigned char*r, unsigned char*g, unsigned char*b);
  void getGrey(int X, int Y, unsigned char*g);
  void getYUV(int X, int Y, unsigned char*y, unsigned char*u, unsigned char*v);

  /* following will set the whole image-data to either black or white
   * the size of the image-data is NOT xsize*ysize*csize but datasize
   * this is mostly slower
   * i have put the datasize into private (like pdata) (bad idea?)
   */
  void setBlack();
  void setWhite();

  /* certain formats are bound to certain csizes,
   * it's quite annoying to have to think again and again (ok, not much thinking)
   * so we just give the format (like GL_LUMINANCE)
   * and it will set the image format to this format
   * and set and return the correct csize (like 1)
   * if no format is given the current format is used
   */
  int setCsizeByFormat(int format);
  int setCsizeByFormat();


  /* various copy functions
   * sometimes we want to copy the whole image (including pixel-data),
   * but often it is enough to just copy the meta-data (without pixel-data)
   * into a new imageStruct
   */
  void copy2Image(imageStruct *to);
  void copy2ImageStruct(imageStruct *to); // copy the imageStruct (but not the actual data)
  /* this is a sort of better copy2Image,
   * which only copies the imageStruct-data if it is needed
   */
  void refreshImage(imageStruct *to);

  /* inplace swapping Red and Blue channel */
  void swapRedBlue    ();


  ///////////////////////////////////////////////////////////////////////////////
  // acquiring data including colour-transformations
  // should be accelerated if possible
  /* i wonder whether this is the right place to put these routines
   * they should be stored somewhere centrally
   * (because maybe a lot of objects would like them) (like [pix_rgba]...)
   * but it might be better to put them (the actual conversion routines) into
   * separate files (a separate library?)
   * orgdata points to the actual data in the given format
   * the datasize will be read from image.xsize, image.ysize
   * the dest-format will be given by image.format
   *   this is maybe not really clean (the meta-data is stored in the destination,
   *   while the source has no meta-data of its own)
   */
  void convertTo  (imageStruct*to,   GLenum dest_format=0);
  void convertFrom(imageStruct*from, GLenum dest_format=0);

  void fromRGB    (unsigned char* orgdata);
  void fromRGBA   (unsigned char* orgdata);
  void fromBGR    (unsigned char* orgdata);
  void fromBGRA   (unsigned char* orgdata);
  void fromRGB16  (unsigned char* orgdata);
  void fromGray   (unsigned char* orgdata);
  void fromGray   (short* orgdata);
  void fromUYVY   (unsigned char* orgdata);
  void fromYUY2   (unsigned char* orgdata); // YUYV
  void fromYVYU   (unsigned char* orgdata);
  /* planar YUV420: this is rather generic and not really YV12 only */
  void fromYV12   (unsigned char* Y, unsigned char*U, unsigned char*V);
  /* assume that the planes are near each other: YVU */
  void fromYV12   (unsigned char* orgdata);
  /* assume that the planes are near each other: YVU */
  void fromYU12   (unsigned char* orgdata);
  /* overloading the above two in order to accept pdp YV12 packets */
  void fromYV12   (short* Y, short*U, short*V);
  void fromYV12   (short* orgdata);

  /* aliases */
  void fromYUV422 (unsigned char* orgdata){fromUYVY(orgdata);}
  void fromYUV420P(unsigned char* orgdata){fromYV12(orgdata);}
  void fromYUV420P(unsigned char*Y,unsigned char*U,unsigned char*V){fromYV12(Y,U,V);}

  // "data" points to the image.
  // the memory could(!) be reserved by this class or someone else
  // "notowned" should be set to "1", if "data" points to foreign memory
  // "data" is not freed directly, when the destructor is called
  unsigned char   *data;    // the pointer to the data
  private:
  // "pdata" is the private data, and is the memory reserved by this class
  // this data is freed when the destructor is called
  unsigned char   *pdata;
  // "datasize" is the size of data reserved at "pdata"
  size_t    datasize;

  public:
  //////////
  // true if the image is flipped horizontally (origin is upper-left)
  // false if the image is openGL-conformant (origin is lower-left)
  GLboolean       upsidedown;

};



// from GemPixUtil.h

struct GEM_EXTERN pixBlock
{
  //pixBlock();

    //////////
    // the block's image
    imageStruct     image;

    //////////
    // is this a newimage since last time?
    //	ie, has it been refreshed
    int     	    newimage;

    //////////
    // keeps track of when new films are loaded
    //	useful for rectangle_textures on OSX
    int		    newfilm;
};

/*-----------------------------------------------------------------
  -------------------------------------------------------------------
  CLASS
  TexCoord


  DESCRIPTION

  -----------------------------------------------------------------*/
class GEM_EXTERN TexCoord
{
 public:
  TexCoord() : s(0.f), t(0.f) { }
    TexCoord(float s_, float t_) : s(s_), t(t_) { }
      float   	    s;
      float   	    t;
};


// from GemState.h


/*-----------------------------------------------------------------
  -------------------------------------------------------------------
  CLASS
  GemState

  The state to pass among GEM objects

  DESCRIPTION

  -----------------------------------------------------------------*/
//struct pixBlock;
//class imageStruct;



class GEM_EXTERN GemState
{
 public:

  //////////
  // Has something changed since the last time?
  int                 dirty;

  //////////
  // Are we in a display list creation?
  int 	    	    inDisplayList;

  //////////
  // Lighting on?
  int 	    	    lighting;

  //////////
  // Smooth shading (flat is other type)
  int 	    	    smooth;

  //////////
  // Texture mapping on?
  // 0..off
  // 1..normalized texture
  // 2..rectangle texture
  int 	    	    texture;

  //////////
  // The image to texture map
  pixBlock	        *image;

  //////////
  // Texture coordinates.
  // This can be NULL if there aren't any coordinates
  TexCoord    	    *texCoords;

  //////////
  // The number of TexCoords
  int 	    	    numTexCoords;

  //////////
  // The number of multiTexUnits
  //   default = 0, max = 7
  int 	    	    multiTexUnits;

  //////////
  // Milliseconds since last frame
  // If in Stereoscopic mode, then it is the same number for both left
  //		and right renderings
  float				tickTime;

  //////////////////
  // the default draw-type (might be overriden within a Geo)
  GLenum				drawType;

  //////////
  // how deep is the current stack /* 4 fields for the 4 stacks */
  int stackDepth[4];

  ////////////
  //vertex-array data
  int                 VertexDirty; // the vertex-arrays has changed

  GLfloat				*VertexArray;
  int					VertexArraySize;
  int					VertexArrayStride;

  GLfloat				*ColorArray;
  int		 			HaveColorArray;

  GLfloat				*NormalArray;
  int					HaveNormalArray;

  GLfloat				*TexCoordArray;
  int					HaveTexCoordArray;

  //////////
  // Constructor
  GemState();

  float texCoordX(int num) {
    if (texture && numTexCoords > num)
      return texCoords[num].s;
    else return 0.;
  }

  float texCoordY(int num) {
    if (texture && numTexCoords > num)
      return texCoords[num].t;
    else return 0.;
  }

  /* reset (parts of?) the GemState: to be called from [gemhead] */
  void reset();

};

// end of #ifndef statement
#endif
